@@ -36,13 +36,13 @@ setClass("dbObjectId", representation(Id = "integer", "VIRTUAL"))
 
 ## coercion methods 
 setAs("dbObjectId", "integer", 
-   def = function(from) as(slot(from,"Id"), "integer")
+   def = function(from) as.integer(slot(from,"Id"))
 )
 setAs("dbObjectId", "numeric",
-   def = function(from) as(slot(from, "Id"), "integer")
+   def = function(from) as.numeric(slot(from, "Id"))
 )
 setAs("dbObjectId", "character",
-   def = function(from) as(slot(from, "Id"), "character")
+   def = function(from) as.character(slot(from, "Id"))
 )   
 
 ## formating, showing, printing,...
@@ -53,12 +53,10 @@ setMethod("format", "dbObjectId",
    valueClass = "character"
 )
 
-setMethod("show", "dbObjectId", def = function(object) print(object))
-
-setMethod("print", "dbObjectId",
-   def = function(x, ...){
-      expired <- if(isIdCurrent(x)) "" else "Expired "
-      str <- paste("<", expired, class(x), ":", format(x), ">", sep="")
+setMethod("show", "dbObjectId",
+   def = function(object){
+      expired <- if (isIdCurrent(object)) "" else "Expired "
+      str <- paste("<", expired, class(object), ":", format(object), ">", sep="")
       cat(str, "\n")
       invisible(NULL)
    }
